/*
 
＜事前準備＞
　1GBのファイルをddコマンドで作成。1回実行すればいい。
 dd if=/dev/urandom of=largefile1GB.dat bs=1M count=1024
 
 javac MemoryFileRead.java でコンパイル

＜実行＞
 OSのキャッシュクリア。
 OSがファイルキャッシュ用にメモリを確保するので一度ファイルキャッシュを空にして実行する
 
  sudo sysctl -w vm.drop_caches=3
  
 java MemoryFileRead で実行
 
*/
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.lang.Thread;

public class MemoryFileRead {
  public static void main(String[] args) {
     // (1)1GBのメモリを確保
     int bufferSize = 1 * 1024 * 1024 * 1024; // 1GB
     ByteBuffer buffer = ByteBuffer.allocate(bufferSize);
     
     try {
        System.out.println("wait for 30 seconds");
        Thread.sleep(30*1000); // 30秒間スリープ
     } catch (InterruptedException e) {
        e.printStackTrace();
     }

     // (2)1GBのファイルを1MB単位で読み取る
     File file = new File("largefile1GB.dat");
     byte[] chunk = new byte[1 * 1024 * 1024]; // 1MB

     try (FileInputStream fis = new FileInputStream(file)) {
       int bytesRead;
       while ((bytesRead = fis.read(chunk)) != -1) {
       }
     } catch (IOException e) {
        e.printStackTrace();
     }
     
     try {
        System.out.println("wait for 60 seconds");
        Thread.sleep(60*1000); // 60秒間スリープ
     } catch (InterruptedException e) {
        e.printStackTrace();
     }
  }
}
