/*

＜事前準備＞
　1GBのファイルをddコマンドで作成。1回実行すればいい。
$ dd if=/dev/urandom of=largefile1GB.dat bs=1M count=1024

ビルド
$ gcc program1.cpp -o program1 -O0

 
＜実行＞
複数実行する場合は、バックジョブで実行する。

./program1 &


最適化による削除

コンパイラの最適化によって、memsetが不要と判断され、削除される可能性があります。
-O0オプションなどで最適化を無効にしてコンパイルする。

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <unistd.h>
#include <time.h>

#define MEMORY_SIZE (2L * 1024 * 1024 * 1024) // 2GB
#define FILE_SIZE (1L * 1024 * 1024 * 1024)   // 1GB
#define CHUNK_SIZE (1L * 1024 * 1024) // 1MB

// 現在時刻を表示する関数
void displayCurrentTime() {
    time_t now = time(NULL);
    struct tm *local = localtime(&now);

    printf("現在の時刻: %02d-%02d-%04d %02d:%02d:%02d\n",
           local->tm_mday, local->tm_mon + 1, local->tm_year + 1900,
           local->tm_hour, local->tm_min, local->tm_sec);
}

int main() {
    pid_t pid = getpid();

    // (1) 2GBのメモリを確保
    displayCurrentTime();
    printf("(%d):",pid);
    printf("malloc =%ld \n",MEMORY_SIZE);
    char *memory = (char *)malloc(MEMORY_SIZE);
    if (memory == NULL) {
        perror("Memory allocation failed");
        return 1;
    }
    // (2) メモリ全体を1MB単位でアクセス
    for (size_t i = 0; i < MEMORY_SIZE; i += CHUNK_SIZE) {
        memset(memory + i, 0xFF, CHUNK_SIZE);
        if(i==0 ){//最初だけ表示。最適化で削除されていないかを確認。
          printf("(%d):",pid);
          printf("Accessed 1MB chunk at offset %zu\n", i);
        }
    }

    //30秒待ち
    printf("(%d):",pid);
    printf("wait for 30 seconds\n");
    sleep(30);

    displayCurrentTime();
    // (3) 1GBのファイルを1MB単位ですべてread
    FILE *file = fopen("largefile1GB.dat", "rb");
    if (file == NULL) {
        perror("File open failed");
        free(memory);
        return 1;
    }

    size_t total_read = 0;
    while (total_read < FILE_SIZE) {
        size_t bytes_to_read = CHUNK_SIZE;
        if (total_read + CHUNK_SIZE > FILE_SIZE) {
           bytes_to_read = FILE_SIZE - total_read;
        }

        size_t bytes_read = fread(memory + total_read, 1, bytes_to_read, file);
        if (bytes_read != bytes_to_read) {
           perror("File read failed");
           fclose(file);
           free(memory);
           return 1;
        }
        total_read += bytes_read;
    }
    displayCurrentTime();

    // (3) 終了せずに待機する (sleepを使用)
    printf("(%d):",pid);
    printf("wait for 60 seconds\n");

    sleep(60);
    displayCurrentTime();

    // 不要になったリソースの解放（実際のプログラムでは、終了時にこれらを解放する必要があります）
    fclose(file);
    free(memory);

    return 0;
}

